/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.configurazione.test;

import java.util.Iterator;
import java.util.Set;

import org.hibernate.Session;

import it.eng.configurazione.bean.ConnetionParameter;
import it.eng.configurazione.bean.DataSource;
import it.eng.configurazione.bean.Interface;
import it.eng.configurazione.bean.SourceType;
import it.eng.configurazione.service.ConfigurationDAO;
import it.eng.configurazione.service.HibernateUtils;
import junit.framework.TestCase;

public class ConnetionParameterTest extends TestCase {

	DataSource dataSourceTest=null;
	SourceType tipo=null;
	Session sessione= HibernateUtils.getSession();
	public ConnetionParameterTest(String name) {
		super(name);
		
	}

	protected void setUp() throws Exception {
		
		ConnetionParameter conP1=new ConnetionParameter();
		conP1.setName("USER");
		conP1.setValue("pippo");
		ConnetionParameter conP2=new ConnetionParameter();
		conP2.setName("PASSWORD");
		conP2.setValue("pippo");		
		
		tipo=new SourceType();
		tipo.setName("JIRA");
		tipo.setDescription("TIPOLOGIA DI SORGENTE JIRA");
		tipo.setClassforname("it.eng.EstrattoreJira.class");
		
		dataSourceTest=new DataSource();
		dataSourceTest.setDescription("Connessione al jira di test");
		dataSourceTest.setName("JIRA_TEST");
		dataSourceTest.setSource(tipo);
		dataSourceTest.addParameter(conP1);
		dataSourceTest.addParameter(conP2);


		sessione.beginTransaction();
		sessione.save(tipo);
		sessione.save(dataSourceTest);
		sessione.getTransaction().commit();
	}

	protected void tearDown() throws Exception {

		sessione.beginTransaction();
		sessione.delete(dataSourceTest);
		sessione.delete(tipo);
		sessione.getTransaction().commit();		
	}

	/**
	 * test di lettura della configurazione
	 *
	 */
	public void testLetturaInterfaccia() {
		ConfigurationDAO dao=new ConfigurationDAO();
		DataSource ds=dao.readDataSource("JIRA_TEST");
		assertEquals(ds.getName(), dataSourceTest.getName());
		assertEquals(ds.getDescription(), dataSourceTest.getDescription());
		assertEquals(ds.getId(), dataSourceTest.getId());
		
		Iterator<ConnetionParameter> iter=ds.getParameters().iterator();
		while (iter.hasNext()){
			ConnetionParameter tmp=iter.next();
			assertTrue(contain(tmp,dataSourceTest.getParameters()));
		}

	}
	
	private boolean contain(ConnetionParameter c, Set<ConnetionParameter> insieme){
		Iterator<ConnetionParameter> iter=insieme.iterator();
		while (iter.hasNext()){
			ConnetionParameter tmp=iter.next();
			if (c.getName().equals(tmp.getName()) && c.getValue().equals(tmp.getValue()))return true;
		}
		return false;
	}

}
