/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.extractors.db;

import it.eng.configurazione.bean.ExtractionProcess;
import it.eng.configurazione.bean.Field;
import it.eng.configurazione.bean.Operation;
import it.eng.extractors.AbstractExtractor;
import it.eng.extractors.ExtratorInterface;
import it.eng.extractors.GenericItem;
import it.eng.extractors.GenericItemInterface;
import it.eng.extractors.GroovyExecutor;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Iterator;

public class DbExtractor extends AbstractExtractor implements ExtratorInterface {

	
	public static final String URL="URL";
	public static final String DRIVER="DRIVER";
	public static final String QUERY="QUERY";
	public static final String USER="USER";
	public static final String PASSWORD="PASSWORD";
	

	private Connection conn=null;
	private ResultSet rs=null;
	private ResultSetMetaData metadata=null;
	private int columnCount=0;
	private boolean hasnext=true;
	
	private Connection openConnection(){
		String driver=operazione.getDataSource().readConnetionParameterValue(DRIVER);
		String url=operazione.getDataSource().readConnetionParameterValue(URL);
		String user=operazione.getDataSource().readConnetionParameterValue(USER);
		String psw=operazione.getDataSource().readConnetionHideParameterValue(PASSWORD);		
		//String psw=operazione.getDataSource().readConnetionParameterValue(PASSWORD);

		try {
			java.lang.Class.forName(driver);
			Connection conn = java.sql.DriverManager.getConnection(url,user,psw);
			return conn;
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}			
		return null;
	}
	
	public boolean hasNext() {
		return hasnext;
	}

	public void init(ExtractionProcess process) {

     	Iterator<Operation> iter=process.getOperation().iterator();
		operazione=iter.next();
		idDataSource=new Integer(operazione.getDataSource().getId());
	    this.conn=openConnection();
	    String query=operazione.readParameterValue(QUERY);	
	    try {
			java.sql.PreparedStatement statement=conn.prepareStatement(query);
			rs= statement.executeQuery();
			metadata = rs.getMetaData();
			columnCount = metadata.getColumnCount();	
		} catch (SQLException e) {
			e.printStackTrace();
		}
	    

	}

	/**
	 * Metodo che legge il valore del RS e applica eventualmente la mappatura.
	 * @return
	 * @throws SQLException
	 */
	private GenericItemInterface createDBInterface() throws SQLException{
		GenericItem ris=new GenericItem();
		ris.setValue("idDataSource", idDataSource);
		Iterator<Field> iterElencoField=operazione.getField().iterator();
		while (iterElencoField.hasNext()) {
			// ciclo sui campi indicati nella configurazione....
			Field campo=iterElencoField.next();
			Object value=estrazioneValore(campo);
			String interfacePosition=campo.getInterfacePosition();
			ris.setValue(interfacePosition, value);
		}
		return ris;
		
	}
	
	/**
	 * Estrae il singolo campo applicando eventualmente la mappatura
	 * @param f
	 * @return
	 * @throws SQLException
	 */
	
	private Object estrazioneValore(Field f) throws SQLException{
		String type=f.getType();
		String name=f.getInterfacePosition();
		Object val=null;
		
		if (type!=null && type.equalsIgnoreCase("String")){
			val= rs.getString(name);
		}
		if (type!=null && type.equalsIgnoreCase("Integer")){
			val= new Integer(rs.getInt(name));
		}
		if (type!=null && type.equalsIgnoreCase("Date")){
			val= rs.getDate(name);
		}

		if ( val!=null && f.getMappingScript()!=null && (val instanceof String)){
			try{
				val=GroovyExecutor.execute(val,f.getMappingScript());
			}catch (Throwable e  ){
				e.printStackTrace();
			}
		}
		return val;
	}
	
	public GenericItemInterface nextItem() {
		try {
			if (rs.next()){
				return createDBInterface();
			}else {
				hasnext=false;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return null;
	}
	

}
