/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.extractors.jira;

import it.eng.configurazione.bean.ExtractionProcess;
import it.eng.configurazione.bean.Field;
import it.eng.configurazione.bean.Operation;
import it.eng.configurazione.service.ConfigurationDAO;
import it.eng.extractors.AbstractExtractor;
import it.eng.extractors.ExtratorInterface;
import it.eng.extractors.GenericItem;
import it.eng.extractors.GenericItemInterface;
import it.eng.extractors.GroovyExecutor;
import it.eng.extractors.jira.rpc.JiraSoapService;
import it.eng.extractors.jira.rpc.JiraSoapServiceService;
import it.eng.extractors.jira.rpc.JiraSoapServiceServiceLocator;
import it.eng.extractors.jira.rpc.RemoteComponent;
import it.eng.extractors.jira.rpc.RemoteCustomFieldValue;
import it.eng.extractors.jira.rpc.RemoteFilter;
import it.eng.extractors.jira.rpc.RemoteIssue;
import it.eng.extractors.jira.rpc.RemoteVersion;

import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.rpc.ServiceException;

import org.apache.commons.beanutils.PropertyUtils;
import org.hibernate.Session;

public class JiraExtractor extends AbstractExtractor implements ExtratorInterface {

	public static final String URL="URL";
	public static final String USER="USER";
	public static final String PASSWORD="PASSWORD";
	public static final String FILTER="FILTER";
	public static final String CUSTOM_FIELD="CUSTOM_FIELD.";
	public static final String KEYSTORE="keyStore";
	public static final String KEYSTORE_PSW="keyStorePSW";
	public static final String TRUSTSTORE="trustStore";
	public static final String TRUSTSTORE_PSW="trustStorePSW";	

	
	private String token=null;
	private JiraSoapService jiraSoapService=null;
	private List<String> progettiDaEstrarre=null;   
	private RemoteFilter remoteFilter=null;
	private String filtro=null;
	private RemoteIssue[] issues=null;
	private int issueNumber=0;
	private int actualIssue=0;
	
	public JiraExtractor(){
	}
	
	
	
	private boolean daEstrarre(String progetto){
		return progettiDaEstrarre.contains(progetto);
	}
	
	
	public void init(ExtractionProcess extractionProcess) {

		Iterator<Operation> iter=extractionProcess.getOperation().iterator();
		operazione=iter.next();
		idDataSource=new Integer(operazione.getDataSource().getId());
		ConfigurationDAO dao=new ConfigurationDAO();
		
		String table=extractionProcess.getInterfaceInfo().getTable();
		//progettiDaEstrarre=dao.readProject(table);
		progettiDaEstrarre=dao.readAllProject();
		
		String url=operazione.getDataSource().readConnetionParameterValue(URL);
		String user=operazione.getDataSource().readConnetionParameterValue(USER);
		String psw=operazione.getDataSource().readConnetionHideParameterValue(PASSWORD);
		
		String keyStore=operazione.getDataSource().readConnetionParameterValue(KEYSTORE);
		String keyStorePsw=operazione.getDataSource().readConnetionHideParameterValue(KEYSTORE_PSW);
		if (keyStore!=null && keyStore.length()>0){
		    System.setProperty("javax.net.ssl.keyStore", keyStore);
		    System.setProperty("javax.net.ssl.keyStorePassword", keyStorePsw);			
		}
		String trustStore=operazione.getDataSource().readConnetionParameterValue(TRUSTSTORE);
		String trustStorePsw=operazione.getDataSource().readConnetionHideParameterValue(TRUSTSTORE_PSW);
		if (keyStore!=null && keyStore.length()>0){
		    System.setProperty("javax.net.ssl.trustStore", trustStore);
		    System.setProperty("javax.net.ssl.trustStorePassword", trustStorePsw); 			
		}

	    
	    try {
	        JiraSoapServiceService jiraSoapServiceGetter = new JiraSoapServiceServiceLocator();
	        URL indirizzo=new URL(url);
	        jiraSoapService = jiraSoapServiceGetter.getJirasoapserviceV2(indirizzo);
	        token = jiraSoapService.login(user, psw);
			filtro=operazione.readParameterValue(FILTER);

			if (filtro==null){
				// gestire errore...
				System.out.println("FILTRO NON IMPOSTATO");
			}else {
		        RemoteFilter[] filtri= jiraSoapService.getSavedFilters(token);
		        for (int i = 0; i < filtri.length; i++) {
		        	RemoteFilter tmp=filtri[i];
		        	if (tmp.getName().equals(filtro)) {
		        		remoteFilter=tmp;
		        		//System.out.println("Filtro trovato:"+filtro);
		        	}
		        }	
		        issues = jiraSoapService.getIssuesFromFilter(token,remoteFilter.getId());
		        issueNumber=issues.length;
			}
						
	    } catch (MalformedURLException e) {
	        e.printStackTrace();
	    } catch (ServiceException e) {
	        e.printStackTrace();
	    }catch (RemoteException e) {
	        e.printStackTrace();
	    }
		
	}

	public GenericItemInterface nextItem() {
		if(actualIssue<issueNumber){
			RemoteIssue issue = issues[actualIssue];
			if (issue!=null && daEstrarre(issue.getProject())) {
				actualIssue++;
				return createIssue(issue);	
			}
			
		}
		return null;

	}

	public boolean hasNext() {
		if (actualIssue<issueNumber) return true;
		else return false;

	}
	
	private GenericItemInterface createIssue(RemoteIssue issue){
		GenericItem risultato =  new GenericItem();
		risultato.setValue("idDataSource", idDataSource);
		risultato.setValue("name", issue.getSummary());
		
		risultato.setValue("description", issue.getDescription());
		risultato.setValue("assegnee", issue.getAssignee());
		risultato.setValue("environment", issue.getEnvironment());
		risultato.setValue("id",issue.getId());	
		risultato.setValue("project",issue.getProject());
		risultato.setValue("priority",applyGroovy("priority",issue.getPriority()));
		risultato.setValue("resolution",applyGroovy("resolution",issue.getResolution()));
		risultato.setValue("state",applyGroovy("state",issue.getStatus()));
		risultato.setValue("type",applyGroovy("type",issue.getType()));
		if (issue.getCreated()!=null && issue.getCreated().getTime()!=null){
			risultato.setValue("creationDate",issue.getCreated().getTime());
		}
		if (issue.getDuedate()!=null && issue.getDuedate().getTime()!=null){
		risultato.setValue("endDate",issue.getDuedate().getTime());
		}
		if (issue.getUpdated()!=null && issue.getUpdated().getTime()!=null){
		risultato.setValue("updatedate",issue.getUpdated().getTime());
		}
		risultato.setValue("notifyBy",issue.getReporter());
		
		if (issue.getFixVersions()!=null ){
			RemoteVersion[] rVersion=issue.getFixVersions();
			if (rVersion!=null && rVersion.length>0){
				risultato.setValue("fixVersions",rVersion[0].getName());
			}else {
				risultato.setValue("fixVersions","");
			}
		}
		if (issue.getAffectsVersions()!=null ){
			RemoteVersion[] rVersion=issue.getAffectsVersions();
			if (rVersion!=null && rVersion.length>0){
				risultato.setValue("affectsVersions",rVersion[0].getName());
			}else {
				risultato.setValue("affectsVersions","");
			}			
		}		
		
		RemoteComponent[] componenti = issue.getComponents();

		// prendo il primo componente
		if (componenti!=null && componenti.length>0){
			RemoteComponent comp = componenti[0];
			risultato.setValue("componentName",comp.getName());	
		}
	
		
		RemoteCustomFieldValue[] customFields = issue.getCustomFieldValues();
		
		Iterator<Field> iterElencoField=operazione.getField().iterator();
		while (iterElencoField.hasNext()) {
			// ciclo sui campi custom indicati nella configurazione....
			Field campo=iterElencoField.next();
			String interfacePosition=campo.getInterfacePosition();
			String key=operazione.readParameterValue(CUSTOM_FIELD+interfacePosition);
			if (key!=null){ // se key == null il campo non  custom
				for (int r = 0; r < customFields.length; r++) {
					RemoteCustomFieldValue comp = customFields[r];
					if (key.equals(comp.getCustomfieldId())){
						// trovato...
						String valore=comp.getValues()[0];
						if (valore != null){
							// impostare l'attributo dell'interfaccia....
							try {
								if (campo.getMappingScript()!=null && campo.getMappingScript().length()>0){
									Object ris=GroovyExecutor.execute(valore,campo.getMappingScript());
									risultato.setValue(interfacePosition,ris.toString() );
								}else {
									risultato.setValue(interfacePosition,valore );							
								}
								
							} catch (Exception e){
								e.printStackTrace();
							}
						}
					}
			    }
			}
			
		}
		return risultato;
	}
	
	

}